/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Interface to ListX
*  SCOPE :              Interface to ListX
*  FILE :               listx.cc
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma implementation
#endif

/* IMPORTS *******************************************************************/

#ifndef __listln_HXX
#include "listln.hxx"
#endif

#ifndef __listx_HXX
#include "listx.hxx"
#endif

#ifndef __iter_HXX
#include "iter.hxx"
#endif

/*****************************************************************************/


/*= EXTERNAL FUNCTION =======================================================*/
void LocalGenListX::init(int elem)
/*=
[DESCRIPTION :]
  Initialise (if needed) a new collection
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if (_col != NULL) return ;
  elem = (elem > ColNbInLeaf) ? ColNbInLeaf : elem ;
  int s = sizeof(Collection) + (elem * sizeof(ColElem)) ;
  PageHdlMem* pd = yoo_getpage(this) ;
  Cluster* clu = (pd != NULL) ? pd->page()->getCluster() : YOO_TMP_CLU ;
  this->_col = (Collection*)ObjectX::operator new(s, clu, YOO_LISTLEAF_CLID,
						 pd==NULL ? NULL : pd->page());
  new(this->_col) ListLeaf(0) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void LocalGenListX::remove(int pos)
/*=
[DESCRIPTION :]
  Remove an element from collection
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->init() ;
  this->domainG(pos) ;
  _col = _col->remove(pos) ;  
}


/*= EXTERNAL FUNCTION =======================================================*/
void* LocalGenListX::getElem(int ind)
/*=
[DESCRIPTION :]
  Return pointer on nth element of the collection
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->domainG(ind) ;
  return this->_col->getElem(ind) ;  
}


/*= EXTERNAL FUNCTION =======================================================*/
int LocalListXObjectX::member(const yoo_ObjectX& obj)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Iterator<ObjectX> iter(*this) ;
  for (; iter.more(); iter.next()) {
    if (iter.get() == obj) {
      return iter.where() ;
    }
  }
  return -1 ;  
}


/*= EXTERNAL FUNCTION =======================================================*/
void LocalListXObjectX::assign(const LocalListXObjectX& l)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->clear() ;
  this->append(l) ; 
}


/*= EXTERNAL FUNCTION =======================================================*/
void LocalListXObjectX::append(const LocalListXObjectX& l)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Iterator<ObjectX> iter(l) ;
  for (; iter.more(); iter.next()) {
    this->insert(iter.get(), COL_TAIL) ;
  }  
}


/*= EXTERNAL FUNCTION =======================================================*/
int LocalListXObjectX::compare(const LocalListXObjectX& l)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if (this->count() != l.count()) return FALSE ;
 
  Iterator<ObjectX> iter1(*this) ;
  Iterator<ObjectX> iter2(l) ;
  for (; iter1.more(); iter1.next(), iter2.next()) {
    if (iter1.get() != iter2.get()) {
      return FALSE ;
    }
  }
  return TRUE ;  
}


/*= EXTERNAL FUNCTION =======================================================*/
void LocalListXObjectX::insert(const yoo_ObjectX& el, int pos)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->init() ;
  this->domainI(pos) ; 
  this->_col = this->_col->insert(&el, pos) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void LocalListX<int>::assign(const LocalListX<int>& l)
/*=
[DESCRIPTION :]
  Verify domain for insertion
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->clear() ;
  *this += l ;
}


/*= EXTERNAL FUNCTION =======================================================*/
LocalListX<int>& LocalListX<int>::operator+=(const LocalListX<int>& l)
/*=
[DESCRIPTION :]
  Verify domain for insertion
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Iterator<int> iter(l) ;
  for (; iter.more(); iter.next()) {
    this->insert(iter.get(), COL_TAIL) ;
  }
  return *this ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int LocalListX<int>::operator==(const LocalListX<int>& l)
/*=
[DESCRIPTION :]
  Verify domain for insertion
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if (this->count() != l.count()) return FALSE ;

  Iterator<int> iter1(*this) ;
  Iterator<int> iter2(l) ;
  for (; iter1.more(); iter1.next(), iter2.next()) {
    if (iter1.get() != iter2.get()) {
      return FALSE ;
    }
  }
  return TRUE ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int LocalListX<int>::member(int el)
/*=
[DESCRIPTION :]
  Verify domain for insertion
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Iterator<int> iter(*this) ;
  for (; iter.more(); iter.next()) {
    if (iter.get() == el) {
      return iter.where() ;
    }
  }
  return -1 ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void LocalListX<double>::assign(const LocalListX<double>& l)
/*=
[DESCRIPTION :]
  Verify domain for insertion
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->clear() ;
  *this += l ;
}


/*= EXTERNAL FUNCTION =======================================================*/
LocalListX<double>& LocalListX<double>::operator+=(const LocalListX<double>& l)
/*=
[DESCRIPTION :]
  Verify domain for insertion
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Iterator<double> iter(l) ;
  for (; iter.more(); iter.next()) {
    this->insert(iter.get(), COL_TAIL) ;
  }
  return *this ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int LocalListX<double>::operator==(const LocalListX<double>& l)
/*=
[DESCRIPTION :]
  Verify domain for insertion
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if (this->count() != l.count()) return FALSE ;
 
  Iterator<double> iter1(*this) ;
  Iterator<double> iter2(l) ;
  for (; iter1.more(); iter1.next(), iter2.next()) {
    if (iter1.get() != iter2.get()) {
      return FALSE ;
    }
  }
  return TRUE ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int LocalListX<double>::member(double el)
/*=
[DESCRIPTION :]
  Verify domain for insertion
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Iterator<double> iter(*this) ;
  for (; iter.more(); iter.next()) {
    if (iter.get() == el) {
      return iter.where() ;
    }
  }
  return -1 ;
}
 

/*= EXTERNAL FUNCTION =======================================================*/
void LocalListX<StringX>::destroy()
/*=
[DESCRIPTION :]
  Verify domain for insertion
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Iterator<StringX> iter(*this) ;
  for (; iter.more(); iter.next()) {
    iter.get().clear() ;
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
void LocalListX<StringX>::assign(const LocalListX<StringX>& l)
/*=
[DESCRIPTION :]
  Verify domain for insertion
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->clear() ;
  *this += l ;
}


/*= EXTERNAL FUNCTION =======================================================*/
LocalListX<StringX>&
LocalListX<StringX>::operator+=(const LocalListX<StringX>& l)
/*=
[DESCRIPTION :]
  Verify domain for insertion
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Iterator<StringX> iter(l) ;
  for (; iter.more(); iter.next()) {
    this->insert(iter.get(), COL_TAIL) ;
  }
  return *this ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int LocalListX<StringX>::operator==(const LocalListX<StringX>& l)
/*=
[DESCRIPTION :]
  Verify domain for insertion
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if (this->count() != l.count()) return FALSE ;

  Iterator<StringX> iter1(*this) ;
  Iterator<StringX> iter2(l) ;
  for (; iter1.more(); iter1.next(), iter2.next()) {
    if (iter1.get() != iter2.get()) {
      return FALSE ;
    }
  }
  return TRUE ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int LocalListX<StringX>::member(const StringX& el)
/*=
[DESCRIPTION :]
  Verify domain for insertion
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Iterator<StringX> iter(*this) ;
  for (; iter.more(); iter.next()) {
    if (iter.get() == el) {
      return iter.where() ;
    }
  }
  return -1 ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void LocalListX<StringX>::insert(char* el, int pos)
/*=
[DESCRIPTION :]
  Verify domain for insertion
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->init() ;
  this->domainI(pos) ;
  void* atom = malloc(sizeof(StringX)) ;
  int size = sizeof(_StringX) + strlen(el) ;
  PageHdlMem* pd = yoo_getpage(this) ;
  Cluster* clu =  pd != NULL ? pd->page()->getCluster() : YOO_TMP_CLU ;
  ((StringX*)atom)->ref =
    (_StringX*)ObjectX::operator new(size, clu, UNTYPED, pd->page()) ;
  strcpy(*((StringX*)atom), el) ;
  this->_col = this->_col->insert(atom, pos) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void LocalListX<StringX>::clear()
/*=
[DESCRIPTION :]
  Verify domain for insertion
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Iterator<StringX> iter(*this) ;
  for (; iter.more(); iter.next()) {
    iter.get().clear() ;
  }

  this->LocalColX::clear() ;
}
